/*
 * botactivity.cpp
 *
 *  Created on: 17 Sep 2015
 *      Author: irina
 */


#include <psconfig.h>
#include "globals.h"


#include "net/connection.h"
#include "util/log.h"
#include "util/strutil.h"
#include "gui/inventorywindow.h"


#include "botactivity.h"

MovementStrategy::MovementStrategy()
{
	size = DEFAULT_SIZE;
	strategy = DEFAULT_STRATEGY;
	startPos = MIDDLE_MAP_POS;
}

void MovementStrategy::Move(int movesDone)
{
	switch (strategy)
	{
	case EXPLORE_RADIAL:
	{
		if (movesDone >0 & movesDone % size ==0)
			if ((movesDone/size) % 2 == 1)
				worldHandler::TurnToPoint(startPos.x, startPos.z);
			else worldHandler::Turn(ANGLE_RADIAL_STRATEGY);
		worldHandler::StepForward();
		break;
	}
	case EXPLORE_GRID:
	{
		worldHandler::StepForward();
		break;
	}
	case EXPLORE_LINE:
	{
		if (((movesDone-1) / size) % 2 ==0)
			worldHandler::StepForward();
		else worldHandler::StepBackward();

		break;
	}
	default: worldHandler::StepForward();
	}
}


ExploreActivity::ExploreActivity()
{
	name = csString("explore");
	textcolour = TEXT_COLOUR;
}
ExploreActivity::~ExploreActivity()
{

}
void ExploreActivity::HandleMessage(MsgEntry* message )
{
	if (!IsOngoing())
		return;

	switch (message->GetType() )
		{
		case MSGTYPE_MODE:
			{
				psModeMessage incoming(message);
				if (incoming.mode == psModeMessage::OVERWEIGHT)
				{
					//Stop();
					OutputMsg(csString("Overweight mode message received."));
				}
				break;
			}
		case MSGTYPE_SYSTEM:
			{
				psSystemMessage incoming(message);
				csString text = incoming.msgline;

				if (text.StartsWith(NOT_SUCCESSFUL))
				{
					noBuild = true;
				}
				else if (text.StartsWith(OVERWEIGHT))
				{
					Stop();
					OutputMsg(csString("Explore stopped because of being overweight."));
				}
				else if (text.StartsWith(DIEING))
				{
					needToDie = true;
				}
				else if (text.StartsWith(EXPLORE_TOOL_WORN))
				{
					noBuild = true;
					OutputMsg(csString("Tool worn-out, will change it."));
					//get a new tool
					csString toolName = worldHandler::GetEquippedToolName();
					worldHandler::Dequip(toolName);
					worldHandler::Equip(toolName);
				}
				else if (text.StartsWith(CLAIM_PLACED))
				{
					noBuild = false;
					size_t end = text.FindLast('!');
					size_t start = CLAIM_PLACED.Length();
					csString thingName = text.Slice(start, end-start-1);
					if (thingName.StartsWith("Tiny"))
						claimType = csString("Tiny");
					else if (thingName.StartsWith("Small"))
						claimType = csString("Small");
					else claimType = csString("Unknown");

					claimName = thingName;

					//and get the item name too if we are at it
					end = claimName.Find(" Exploration Marker");
					start = claimType.Length()+1;
					size_t length = end - start;
					itemName = claimName.Slice(start, length);
				}
				else if (text.StartsWith(GOT_ITEMS))
				{
					size_t end = text.FindLast('!');
					size_t start = GOT_ITEMS.Length();
					csString thingName = text.Slice(start, end-start);

					if (thingName.StartsWith(ITEM_KEY))
					{
						keyName = thingName;
					}
					else
					if (thingName.EndsWith(ITEM_ENUM))
					{
						scrollName = thingName;
					}
				}
				break;
			}
		case MSGTYPE_PERSIST_ITEM:
			{
				psPersistItem incoming(message, psengine->GetNetManager()->GetConnection()->GetAccessPointers());
				message->Reset();
				if (incoming.name.EndsWith(EXP_MARKER))
				{
					markerID = incoming.eid;
				}
				break;
			}
		}
}
void ExploreActivity::ScheduleNewPerform(int delay)
{
	csRef<iEventTimer> timer = worldHandler::GetStandardTimer();
	timer->AddTimerEvent(this, delay);
}
bool ExploreActivity::Perform(iTimerEvent *ev)
{

	if (!IsOngoing() && !IsFinished())
		return false;

	switch (actionToDo)
	{
		case ACTION_INIT:
		{
			DoInit();
			NextAction(MIN_DELAY);
			break;
		}
		case ACTION_EXPLORE:
		{
			DoExplore();
			NextAction(maxDelay);
			break;
		}
		case ACTION_BUILD:
		{
			DoBuild();
			NextAction(MIN_DELAY);
			break;
		}
		case ACTION_USE_COMBINE:
		{
			DoUseCombine();
			NextAction(maxDelay);
			break;
		}
		case ACTION_TAKE_RESULT:
		{
			DoTakeResult();
			NextAction(MIN_DELAY);
			break;
		}
		case ACTION_DIE:
		{
			DoDie();
			NextAction(DEATH_DELAY);
			break;
		}
		case ACTION_MOVE:
		{
			DoMove();
			NextAction(MIN_DELAY);
			break;
		}

		case ACTION_STOP:
		{
			DoStop();
			NextAction(MIN_DELAY);
			break;
		}
		case ACTION_NONE:
		{
			Finish();
			break;
		}
	}
	return false;
}

void ExploreActivity::StartWork()
{
	OutputMsg(csString("Explore activity started."));
	ScheduleNewPerform(MIN_DELAY);
}
void ExploreActivity::WrapUp()
{
	csVector3 pos = worldHandler::GetCurrentPos();
	csString sep = csString(",");
	char x[100], y[100], z[100];
	sprintf(x, "%.2f", pos.x);
	sprintf(y, "%.2f", pos.y);
	sprintf(z, "%.2f", pos.z);

	char itemCount[100];
	int itemsCountFinal = itemsCountAfter-itemsCountBefore;
	sprintf(itemCount, "%d", itemsCountFinal);

	csString toolName = worldHandler::GetEquippedToolName();
	csString text = toolName +sep+csString(x)+sep+csString(y)+sep+csString(z)+sep+claimName+sep+csString(itemCount);
	if (logger == NULL)
		OutputMsg(csString("No logger available, so activities will NOT be logged."));
	else
		logger->Write(CSV_EXPLORING, text);

	if (timesDone>=timesToRepeat)
	{
		OutputMsg(csString("Explore activity finished."));
	}
	else
	{
		char msg[1000];
		sprintf(msg, "%d explore attempts done, %d attempts left.", timesDone, timesToRepeat-timesDone);
		OutputMsg(csString(msg));
	}
}
void ExploreActivity::DoInit()
{
	actionToDo = ACTION_INIT;
	attemptsCount = 0;
	claimName = csString("");
	scrollName = csString("");
	claimType = csString("Unknown");
	keyName = csString("");
	markerID = 0;

	itemsCountBefore = 0;
	itemsCountAfter = 0;
	itemName = csString("");

	needToDie = false;
	noBuild = true;

	int sz;
	EXPLORE_STRATEGIES strat;

	if (timesDone == 0)
	{
		WordArray words(cmd,false);

		strat = EXPLORE_LINE;

		if (words.GetCount()>0)
			sz = words.GetInt(0);
		else sz = DEFAULT_SIZE;

		if (sz == 0)
		{
			sz = DEFAULT_SIZE;
			OutputMsg(csString("Size given as 0, will work instead with default size!"));
		}

		if (words.GetCount() > 1)
			if (words.GetInt(1) == 0)
				{
					leaveKeys = false;
					OutputMsg(csString("Will keep keys."));
				}
			else
				{
					leaveKeys = true;
					OutputMsg(csString("Will leave keys in claim."));
				}

		if (words.GetCount() > 2)
			smallClaimsCount = words.GetInt(2);
		else smallClaimsCount = DEFAULT_THREADS_SMALL_CLAIM;

		if (words.GetCount() > 3)
			maxDelay = words.GetInt(3);
		else maxDelay = MAX_DELAY;
	}

	if (timesDone == 0)
		strategy.Init(strat, sz, startPoint, startYRot);
}

void ExploreActivity::NextAction(int delay)
{
	switch (actionToDo)
	{
		case ACTION_INIT:
		{
			if (timesDone >= timesToRepeat)
				{
					actionToDo = ACTION_NONE;
				}
			else
				actionToDo = ACTION_EXPLORE;
			break;
		}
		case ACTION_EXPLORE:
		{
			actionToDo = ACTION_BUILD;
			break;
		}
		case ACTION_BUILD:
		{
			if (noBuild)
				if (needToDie)
					actionToDo = ACTION_DIE;
				else actionToDo = ACTION_MOVE;
			else
				actionToDo = ACTION_USE_COMBINE;
			break;
		}
		case ACTION_USE_COMBINE:
		{
			actionToDo = ACTION_TAKE_RESULT;
			break;
		}
		case ACTION_TAKE_RESULT:
		{
			if (needToDie)
				actionToDo = ACTION_DIE;
			else actionToDo = ACTION_MOVE;
			break;
		}
		case ACTION_MOVE:
		{
			actionToDo = ACTION_STOP;
			break;
		}
		case ACTION_STOP:
		{
			actionToDo = ACTION_INIT;
			break;
		}
		case ACTION_DIE:
		{
			actionToDo = ACTION_MOVE;
			break;
		}
		default:
		{
			actionToDo = ACTION_NONE;
			return;
		}
	}
	ScheduleNewPerform(delay);
}

void ExploreActivity::DoExplore()
{
	//get free slot where the key will likely go if a claim is found
	pawsInventoryWindow* inv = (pawsInventoryWindow*)PawsManager::GetSingleton().FindWidget("InventoryWindow");
	nextFreeSlot = inv->GetFreeSlot();

	//do the actual explore
	worldHandler::ExecCmd(csString("/explore"));
}

void ExploreActivity::DoBuild()
{
	if (!noBuild && !itemName.CompareNoCase("") && markerID != 0 && !scrollName.CompareNoCase("") && !claimName.CompareNoCase(""))
	{
		//we got everything, so we got a claim supposedly
		if (claimType.CompareNoCase("Unknown"))
		{
			//don't know how to build this, so just lock it and move on
			worldHandler::PicklockEID(markerID);
			noBuild = true;
			return;
		}

		psInventoryCache::CachedItemDescription* itemSlot = worldHandler::FindItemSlot(itemName, false);
		if (itemSlot == NULL)
			itemsCountBefore = 0;
		else itemsCountBefore = itemSlot->stackCount;

		itemsCountAfter = itemsCountBefore;		//as init

		worldHandler::TargetEID(markerID);
		csString targetName = worldHandler::GetTargetName();
		worldHandler::OpenTargetEID(markerID);
		int toContainer = markerID.Unbox();

		if (!worldHandler::IsInBrain(scrollName))
		{
			if (!worldHandler::EquipRecipe(scrollName))
		         {
					OutputMsg(csString("Missing blueprint for the claim!"));
					noBuild = true;
		            //Error();
		            return;
		         }
		}

		int quantity=0;
		csString item;
		if (claimType.CompareNoCase("Tiny"))
		{
			quantity = ITEM_COUNT_TINY_CLAIMS;
			item = ITEM_NAME_FOR_TINY_CLAIMS;
		}
		else if (claimType.CompareNoCase("Small"))
		{
			quantity = smallClaimsCount;
			item = ITEM_NAME_FOR_SMALL_CLAIMS;
		}
		else
			{
				noBuild = true;
				return;
			}

		psInventoryCache::CachedItemDescription* from = worldHandler::FindItemSlot(item, false);

		if (!from || from->stackCount < quantity)
		{
			//not enough items to build claim, so just move on (don't lock tinies and smalls!!!)
			noBuild = true;
			return;
		}

		worldHandler::MoveItems(from->containerID, from->slot, toContainer, 0, quantity);
		worldHandler::TargetEID(markerID);
	}
	else
		noBuild = true;
}
void ExploreActivity::DoUseCombine()
{
	worldHandler::TargetEID(markerID);
	if (claimType.CompareNoCase("Tiny"))
		worldHandler::UseTarget();
	else if (claimType.CompareNoCase("Small"))
		worldHandler::CombineContentsInTarget();
}
void ExploreActivity::DoTakeResult()
{
	worldHandler::TargetEID(markerID);
	worldHandler::TakeAllFromTarget();

	//move key to container
	if (leaveKeys && nextFreeSlot!=NULL && nextFreeSlot->GetToolTip().CompareNoCase("Key"))
	{
		int container   = nextFreeSlot->ContainerID();
		int slot        = nextFreeSlot->ID();
		psSlotMovementMsg msg( container, slot, markerID.Unbox(), 0, 1);
		msg.SendMessage();
	}
}
void ExploreActivity::DoDie()
{
	worldHandler::ExecCmd(csString("/die"));
	needToDie = false;
}
void ExploreActivity::DoMove()
{
	//for logging purpose; counted here so that all loot has time to arrive
	if (!itemName.CompareNoCase(""))
	{
		psInventoryCache::CachedItemDescription* itemSlot = worldHandler::FindItemSlot(itemName, false);
		if (itemSlot != NULL)
			itemsCountAfter = itemSlot->stackCount;
		else itemsCountAfter = 0;
	}
	else {
		itemsCountAfter = 0;
		itemsCountBefore = 0;
	}

	strategy.Move(timesDone);
}
void ExploreActivity::DoStop()
{
	worldHandler::StopAllMovement();
	Finish();
}

/*************************Craft Activity*****************/

CraftActivity::CraftActivity()
{
	name = csString("craft");
	textcolour = TEXT_COLOUR;
}
CraftActivity::~CraftActivity()
{

}
void CraftActivity::HandleMessage(MsgEntry* message )
{
	if (!IsOngoing())
		return;

	switch (message->GetType() )
	{
		case MSGTYPE_MODE:
		{
			break;
		}
		case MSGTYPE_GUISKILL:
		{
			craftFinished = true;
			break;
		}
		case MSGTYPE_SYSTEM:
		{
			psSystemMessage incoming(message);
			csString text = incoming.msgline;

			if (text.StartsWith(RANKING_PREFIX))
			{
				needToTrain = true;
				size_t end = text.FindLast('!');
				size_t start = text.FindLast(' ');
				skillToTrain = text.Slice(start+1, end-start-1);
			}
			else if (text.StartsWith(CONTAINER_WORN))
			{
				Error();
				OutputMsg("Container worn-out, crafting will stop.");
			}
			else if (text.StartsWith(TOOL_WORN))
			{
				Error();
				OutputMsg("Tool worn-out, crafting will stop.");
			}
			else if (text.StartsWith(NOT_IN_RANGE))
			{
				Error();
				OutputMsg("Can't seem to find the right container, crafting will stop.");
			}
			break;
		}
		case MSGTYPE_CRAFT_INFO:
		{
			psMsgCraftingInfo incoming(message);
			csString itemDescription(incoming.craftInfo);

			rcp.ParseRecipe(itemDescription, useMax);
			worldHandler::CloseOpenWindow(csString("Read Book"));
			OutputMsg(csString("Done reading recipe."));
			break;
		}
	}
}
void CraftActivity::ScheduleNewPerform(int delay)
{
	csRef<iEventTimer> timer = worldHandler::GetStandardTimer();
	timer->AddTimerEvent(this, delay);
}
bool CraftActivity::Perform(iTimerEvent *ev)
{

	if (!IsOngoing() && !IsFinished())
		return false;

	switch (actionToDo)
	{
		case CRAFTING_INIT:
		{
			DoInit();
			NextAction(MIN_DELAY);
			break;
		}
		case CRAFTING_READRECIPE:
		{
			DoReadRecipe();
			NextAction(MIN_DELAY);
			break;
		}
		case CRAFTING_USE_COMBINE:
		{
			DoCombine();
			NextAction(craftDelay);
			break;
		}
		case CRAFTING_TAKE_RESULT:
		{
			if (craftFinished)
			{
				DoTakeResult();
				NextAction(MIN_DELAY);
			}
			else
			{
				craftDelay = craftDelay + MIN_DELAY;
				OutputMsg(csString("Item not yet ready, will wait some more and update craft delay."));
				return true;
			}
			break;
		}
		case CRAFTING_TRAIN:
		{
			DoTrain();
			NextAction(MIN_DELAY);
			break;
		}
		case CRAFTING_NONE:
		{
			OutputMsg(csString("Craft activity finished."));
		}
	}
	return false;
}

void CraftActivity::StartWork()
{
	OutputMsg(csString("Craft activity started."));
	ScheduleNewPerform(MIN_DELAY);
}
void CraftActivity::WrapUp()
{
	if (logger == NULL)
		OutputMsg(csString("No logger available, so activities will NOT be logged."));
	else
		logger->Write(CSV_CRAFTING, logText);
}
void CraftActivity::DoInit()
{
	needToTrain = false;
	craftFinished = false;
	logText = csString("");
	trainSubstage = 0;


	if (timesDone == 0)// && !cmd.CompareNoCase(""))
	{
		WordArray words(cmd,false);

		if (words.GetCount() > 0)
			craftDelay = words.GetInt(0);
		else craftDelay = MAX_DELAY;

		if (words.GetCount() > 1 && words[1] == "m")
			  useMax = false;
		else useMax = true;

		csString recipeName = worldHandler::GetBrainItemName();
		if (recipeName.CompareNoCase(""))
		{
			Error();
			actionToDo = CRAFTING_NONE;
			OutputMsg(csString("No blueprint equipped, please equip the bp for what you want to craft and /bot craft again."));
			return;
		}
		else
			rcp.setRecipeName(recipeName);
	}
//	else if (timesDone == 0)
//	{
//		useMax = true;
//		craftDelay = MAX_DELAY;
//	}


	if (useMax)
		logText = rcp.GetItemName()+LOG_TEXT_SEP+"Max"+LOG_TEXT_SEP;
	else logText = rcp.GetItemName()+LOG_TEXT_SEP+"min"+LOG_TEXT_SEP;

	actionToDo = CRAFTING_READRECIPE;
}

void CraftActivity::NextAction(int delay)
{
	switch (actionToDo)
	{
	case CRAFTING_INIT:
	{
		actionToDo = CRAFTING_READRECIPE;
		break;
	}
	case CRAFTING_READRECIPE:
	{
		if (timesDone < timesToRepeat)
			actionToDo = CRAFTING_USE_COMBINE;
		else actionToDo = CRAFTING_NONE;
		break;
	}
	case CRAFTING_USE_COMBINE:
	{
		actionToDo = CRAFTING_TAKE_RESULT;
		break;
	}
	case CRAFTING_TAKE_RESULT:
	{
		if (needToTrain)
			actionToDo = CRAFTING_TRAIN;
		else actionToDo = CRAFTING_INIT;
		break;
	}
	case CRAFTING_TRAIN:
	{
		if (!needToTrain)
			actionToDo = CRAFTING_INIT;
		break;
	}
	}
	ScheduleNewPerform(delay);
}

void CraftActivity::MoveToContainer()
{
	OutputMsg(csString("Moving items to container for training..."));

	if (!worldHandler::Target(rcp.GetContainerName()))
	{
		OutputMsg(csString("Could not find container within reach!"));
		Error();
		return;
	}

	if (!worldHandler::OpenTarget())
	{
		OutputMsg(csString("Could not open the container!"));
		Error();
		return;
	}

	int toContainer = worldHandler::GetTargetID();
	if (toContainer < 0)
	{
		OutputMsg(csString("Lost target on container, please /bot craft again if the container is truly available and within reach."));
		Error();
		return;
	}


	//move recipe and ingredients from inventory to container to avoid overweight issues

	csHash<int, csString>::GlobalIterator iterIngredients(rcp.GetIngredientsList()->GetIterator());

	int nextEmptySlot = 0;
	if (!iterIngredients.HasNext())
	{
		OutputMsg(csString("Empty ingredients list!"));
		Error();
		return;
	}
	while (iterIngredients.HasNext())
	{
		csString itemName;
		iterIngredients.Next(itemName);

		psInventoryCache::CachedItemDescription* from = worldHandler::FindItemSlot(itemName, false);
		if (from)
		{
			worldHandler::MoveItems(from->containerID, from->slot, toContainer, nextEmptySlot, from->stackCount);
		}
		nextEmptySlot = nextEmptySlot + 1;
	}
	//move recipes too
	psInventoryCache::CachedItemDescription * from = worldHandler::FindItemSlot(rcp.GetRecipeName(), false);
	if (from)
		worldHandler::MoveItems(from->containerID, from->slot, toContainer, nextEmptySlot, from->stackCount);

	OutputMsg(csString("Done moving items to container."));
}

void CraftActivity::DoReadRecipe()
{
	psViewItemDescription out(CONTAINER_INVENTORY_EQUIPMENT, PSCHARACTER_SLOT_MIND);
	out.SendMessage();
	OutputMsg(csString("Reading the recipe..."));
}
void CraftActivity::DoCombine()
{
	OutputMsg(csString("Craft action combining..."));
	char out[200];
	sprintf(out, "Done %d items, %d left to do.", timesDone, timesToRepeat-timesDone);
	OutputMsg(csString(out));

	if (!worldHandler::Target(rcp.GetContainerName()))
	{
		OutputMsg(csString("Could not find container within reach!"));
		Error();
		return;
	}

	if (!worldHandler::OpenTarget())
	{
		OutputMsg(csString("Could not open the container!"));
		Error();
		return;
	}

	int toContainer = worldHandler::GetTargetID();
	if (toContainer < 0)
	{
		OutputMsg(csString("Lost target on container, please /bot craft again if the container is truly available and within reach."));
		Error();
		return;
	}

	OutputMsg(csString("Moving ingredients to container for crafting"));
	//move ingredients from inventory to container
	//first check for any bundles

	psInventoryCache::CachedItemDescription* from = worldHandler::FindItemSlot(rcp.GetBundleName(), false);
	if (from)
	{
		worldHandler::MoveItems(from->containerID, from->slot, toContainer, 0, 1);
		worldHandler::UseTarget();
		logText = logText + rcp.GetBundleName();
		return;
	}
	csHash<int, csString>::GlobalIterator iterIngredients(rcp.GetIngredientsList()->GetIterator());

	int nextEmptySlot = 0;
	if (!iterIngredients.HasNext())
	{
		OutputMsg(csString("Empty ingredients list!"));
		Error();
		return;
	}
	while (iterIngredients.HasNext())
	{
		csString itemName;
		int quantity = iterIngredients.Next(itemName);

		char out[1000];
		sprintf(out, "Ingredient %s: %d", itemName.GetData(), quantity);
		OutputMsg(csString(out));

		from = worldHandler::FindItemSlot(itemName, false);
		if (!from || from->stackCount < quantity)
		{
			OutputMsg(csString("Not enough ingredients for crafting! Bot stopping."));
			Error();
			return;
		}
		else
		{
			worldHandler::MoveItems(from->containerID, from->slot, toContainer, nextEmptySlot, quantity);
			logText = logText+csString(out)+ csString(" ");
		}
		nextEmptySlot = nextEmptySlot + 1;
	}
	OutputMsg(csString("Done with ingredients. Starting to combine."));

	worldHandler::CombineContentsInTarget();
}
void CraftActivity::DoTakeResult()
{
	OutputMsg(csString("Craft action taking result..."));
	worldHandler::TakeAllFromTarget();

	Finish();
}
void CraftActivity::DoTrain()
{
	if (needToTrain)
	{
		if (trainSubstage == 0)
		{
			MoveToContainer();	//to avoid overweight issue
			trainSubstage = trainSubstage + 1;
		}
		else if (trainSubstage == 1)
		{
			worldHandler::TrainSkillMax(skillToTrain);
			trainSubstage = trainSubstage + 1;
		}
		else
		{
			if (!worldHandler::Target(rcp.GetContainerName()))
			{
				OutputMsg(csString("Could not find container within reach!"));
				Error();
				return;
			}
			worldHandler::TakeAllFromTarget();
			worldHandler::EquipRecipe(rcp.GetRecipeName());

			needToTrain = false;
		}
	}
}
