/*
 * botactivity.h
 *
 *  Created on: 17 Sep 2015
 *      Author: irina
 */

#ifndef BOTACTIVITY_H_
#define BOTACTIVITY_H_

#include "net/message.h"
#include "botlog.h"
#include <csutil/ref.h>
#include "worldhandler.h"
#include "botstorage.h"
#include "recipe.h"

#include "gui/pawsslot.h"


//generic states for any bot activity
enum ACTIVITY_STATES
{
	ERROR_ = -1,
	ONGOING_ = 0,
	FINISHED_ = 1,
	STOPPED_ = 2,
	READY_
};
//explore strategies for the explore activity
enum EXPLORE_STRATEGIES
{
	EXPLORE_RADIAL = 1,
	EXPLORE_GRID = 2,
	EXPLORE_LINE = 3
};

//step-wise actions done by the crafting activity between imposed/forced delays
enum CRAFT_ACTIONS
{
	//CRAFTING_LEARNSTORAGE = 21,
	CRAFTING_INIT = 21,
	CRAFTING_READRECIPE = 22,
	//CRAFTING_GETINGREDIENTS = 23,
	CRAFTING_USE_COMBINE = 24,
	CRAFTING_TAKE_RESULT = 25,
	CRAFTING_TRAIN = 26,
	CRAFTING_NONE
};

//step-wise actions done by the explore activity between imposed/forced delays
enum EXPLORE_ACTIONS
{
	ACTION_INIT = 10,
	ACTION_EXPLORE = 11,
	ACTION_BUILD = 12,
	ACTION_USE_COMBINE = 13,
	ACTION_TAKE_RESULT = 14,
	ACTION_MOVE = 15,
	ACTION_STOP = 16,
	ACTION_DIE = 17,
	ACTION_NONE
};

const int MAX_DELAY = 8000;
const int MIN_DELAY = 600;
const int DEATH_DELAY = 42000;
const int DEFAULT_SIZE = 3;
const int DEFAULT_THREADS_SMALL_CLAIM = 7;
const int ITEM_COUNT_TINY_CLAIMS = 1;
const int TEXT_COLOUR = 61046;
const int MAX_ATTEMPTS = 4;



const int ANGLE_RADIAL_STRATEGY = 5;
const EXPLORE_STRATEGIES DEFAULT_STRATEGY = EXPLORE_RADIAL;
const csVector3 MIDDLE_MAP_POS = csVector3(0, 52.26, 0);
const csVector3 TRAINER_POS = csVector3(174, 57, 159);
static const csString EXP_MARKER = csString("Exploration Marker");
static const csString NOT_SUCCESSFUL = csString("You were not successful");
static const csString EXPLORE_TOOL_WORN = csString("That tool is not useful");
static const csString CLAIM_PLACED = csString("You placed an ");
static const csString GOT_ITEMS = csString("You got a ");
static const csString ITEM_KEY = csString("Key");
static const csString ITEM_ENUM = csString("Ennumeration");
static const csString DIEING = csString("Your hands are raw, your knees buckle");
static const csString OVERWEIGHT = csString("You struggle under the weight of your inventory and must drop something");
static const csString RANKING_PREFIX = csString("You've ranked up");
static const csString CONTAINER_WORN = csString("The container is too worn to be useful");
static const csString TOOL_WORN = csString("You do not have a good tool");
static const csString NOT_IN_RANGE = csString("You are not in range to ");

//generic class for bot activities - to create a new activity, inherit from this class
class BotActivity
{
public:

	virtual void HandleMessage(MsgEntry* message ) =0;

	csString GetActivityName() {
		return name;
	}

	void OutputMsg(csString msg)
	{
		if (textbox != NULL)
		{
			csString out = csString(">") + msg;
			textbox->Show();
			textbox->AddMessage(out.GetData(), textcolour);
		}
	}

	void Init(BotLogCSV* logger, int timesToRepeat, pawsMessageTextBox* textbox, csString cmd) {
		this->logger = logger;
		this->timesToRepeat = timesToRepeat;
		this->textbox = textbox;
		timesDone = 0;
		startPoint = worldHandler::GetCurrentPos();
		startYRot = worldHandler::GetCurrentRot();
		this->cmd = cmd;

		DoInit();
		state = READY_;
	}
	void Start() {
		state = ONGOING_;
		StartWork();
	}
	void Stop() {
		state = STOPPED_;
		WrapUp();
	}
	void Finish() {
		timesDone = timesDone + 1;
		WrapUp();
		state = FINISHED_;
	}
	void Restart() {
		Stop();
		Start();
	}
	void Error() {
		state = ERROR_;
		WrapUp();
	}

	void Reset() {
		Stop();
		Init(logger, timesToRepeat, textbox, cmd);
	}

	bool IsOngoing() {
		return state == ONGOING_ || (state == FINISHED_ && timesDone < timesToRepeat);
	}

	bool IsError() {
		return state == ERROR_;
	}

	bool IsFinished() {
		return state == FINISHED_;
	}

	bool IsStopped() {
		return state == STOPPED_;
	}
	bool IsReady() {
		return state == READY_;
	}

	virtual void ScheduleNewPerform(int delay) =0;

private:
	ACTIVITY_STATES state;

protected:
	csString name;		//name of activity, intended to be set by classes inheriting this one

    pawsMessageTextBox* textbox;
    int textcolour;
    csString cmd;

	int timesToRepeat;
	int timesDone;
	csVector3 startPoint;
	float startYRot;
	csString skillToTrain;

	virtual void StartWork() =0;
	virtual void WrapUp() =0;
	virtual void DoInit() =0;

	BotLogCSV* logger;
};


class MovementStrategy
{
public:
	MovementStrategy();
	void Move(int movesDone);
	void Init(EXPLORE_STRATEGIES strategy, int size, csVector3 startPos, float yRot)
	{
		this->strategy = strategy;
		this->size = size;
		this->startPos = startPos;
		this->startYRot = yRot;
	}

protected:
	EXPLORE_STRATEGIES strategy;
	int size;						//basically a measure of the area to cover

	csVector3 startPos;
	float startYRot;
private:
	csVector3 currPos;
};

class ExploreActivity: public BotActivity, iTimerEvent
{
public:
	ExploreActivity();
	~ExploreActivity();
	void HandleMessage(MsgEntry* message );
	void ScheduleNewPerform(int delay);
	bool Perform(iTimerEvent *ev);

	//void SetStrategy(MovementStrategy ms);
	MovementStrategy GetStrategy() {
		return strategy;
	}

	//for iBase
			void IncRef(){}
			void DecRef(){}
			int GetRefCount() {return 0;}
			void* QueryInterface(scfInterfaceID i, int a) {}
			void AddRefOwner(void **, CS::Threading::Mutex*) {}
			void RemoveRefOwner(void**){}
			scfInterfaceMetadataList* GetInterfaceMetadata() {return NULL;}

protected:

	void StartWork();
	void WrapUp();
	void DoInit();

	void NextAction(int delay);
	//action methods

	void DoExplore();
	void DoBuild();
	void DoUseCombine();
	void DoTakeResult();
	void DoDie();
	void DoMove();
	void DoStop();


private:
	int attemptsCount;
	csString claimName;
	csString scrollName;
	csString keyName;
	EID markerID;
	csString claimType;
	int itemsCountBefore;
	int itemsCountAfter;
	csString itemName;

	int smallClaimsCount;
	int maxDelay;

	//flags
	bool needToDie;
	bool noBuild;
	bool leaveKeys;

	MovementStrategy strategy;

	pawsSlot* nextFreeSlot;		//for getting rid of tiny+small keys


	EXPLORE_ACTIONS actionToDo;
};

class CraftActivity: public BotActivity, iTimerEvent
{
public:
	CraftActivity();
	~CraftActivity();
	void HandleMessage(MsgEntry* message );
	void ScheduleNewPerform(int delay);
	bool Perform(iTimerEvent *ev);

	//for iBase
			void IncRef(){}
			void DecRef(){}
			int GetRefCount() {return 0;}
			void* QueryInterface(scfInterfaceID i, int a) {}
			void AddRefOwner(void **, CS::Threading::Mutex*) {}
			void RemoveRefOwner(void**){}
			scfInterfaceMetadataList* GetInterfaceMetadata() {return NULL;}

protected:

	void StartWork();
	void WrapUp();
	void DoInit();

	void NextAction(int delay);
	void MoveToContainer();

	//actions methods
	void DoReadRecipe();
	void DoCombine();
	void DoTakeResult();
	void DoTrain();

private:
	bool needToTrain;
	int craftDelay;
	bool useMax;
	bool craftFinished;
	csString logText;
	int trainSubstage;

	recipe rcp;

	CRAFT_ACTIONS actionToDo;
};

#endif /* BOTACTIVITY_H_ */
