#include "botlog.h"
#include "util/log.h"

BotLogCSV::BotLogCSV(iConfigManager* configmanager, iVFS* vfs)
{
    std::pair<csString, const char*> logs[MAX_BOTCSV];
    size_t maxSize = 10*1024*1024;

    logs[CSV_RECIPES] = std::make_pair(csString("logrecipe.csv"), RECIPE_HEADER);
    logs[CSV_EXPLORING] = std::make_pair(csString("logexploring.csv"), EXPLORING_HEADER);
    logs[CSV_CRAFTING] = std::make_pair(csString("logcrafting.csv"), CRAFTING_HEADER);
    logs[CSV_POS] = std::make_pair(csString("logpos.csv"), POS_HEADER);

    for(int i = 0;i < MAX_BOTCSV;i++)
    {
        StartLog(logs[i].first, vfs, logs[i].second, maxSize, csvFile[i]);
    }
}

void BotLogCSV::StartLog(const char* logfile, iVFS* vfs, csString header, size_t maxSize, csRef<iFile>& csvFile)
{
	csString filename;
	filename.Format("/planeshift/userdata/logs/%s_%s",
	        	                        psengine->GetMainPlayerName(),
	        	                        logfile);
	filename.ReplaceAll(" ", "_");

        bool writeHeader = false;
        if (!vfs->Exists(filename))
        {

            csvFile = vfs->Open(filename,VFS_FILE_WRITE);
            writeHeader = true;
        }
        else
        {
            csvFile = vfs->Open(filename,VFS_FILE_APPEND);

            // Need to rotate log
            if (csvFile && csvFile->GetSize() > maxSize)
            {
               // CPrintf(CON_ERROR, "Log File %s is too big! Current size is: %u. Rotating log.", logfile, csvFile->GetSize());

                csvFile = NULL;

                // Rolling history
                for (int index = 10; index > 0; index--)
                {
                    csString src(logfile), dst(logfile);
                    src.Append(index);
                    dst.Append(index + 1);
                    // Rotate the files (move file[index] to file[index+1])
                    if (vfs->Exists(src))
                    {
                        csRef<iDataBuffer> existingData = vfs->ReadFile(src, false);
                        vfs->WriteFile(dst, existingData->GetData(), existingData->GetSize());
                    }
                }

                csRef<iDataBuffer> existingData = vfs->ReadFile(logfile, false);

                csString temp(logfile);
                vfs->WriteFile(temp + "1", existingData->GetData(), existingData->GetSize());
                csvFile = vfs->Open(logfile,VFS_FILE_WRITE);

                writeHeader = true;
            }
        }
        if(csvFile.IsValid() && writeHeader)
        {
            csvFile->Write(header.GetData(), header.Length());
            csvFile->Flush();
        }
}

void BotLogCSV::Write(int type, csString& text)
{
    if (!csvFile[type])
        return;


    time_t curtime = time(NULL);
    struct tm *loctime;
    loctime = localtime (&curtime);
    csString buf(asctime(loctime));
    buf.Truncate(buf.Length()-1);

    buf.Append(", ");
    buf.Append(text);
    buf.Append("\n");

    csvFile[type]->Write(buf, buf.Length());
    csvFile[type]->Flush();

}
