/*
 * foxybot.cpp
 *
 *  Created on: 22 Jul 2015
 *      Author: Diana Coman
 */
#include <psconfig.h>
#include "globals.h"

#include "psclientchar.h"
#include "gui/inventorywindow.h"
#include "pscelclient.h"


// PAWS INCLUDES

#include "net/messages.h"
#include "net/clientmsghandler.h"
#include "util/log.h"
#include "net/connection.h"

#include "net/cmdhandler.h"

#include "foxybot.h"
#include "gui/pawsslot.h"
#include "util/slots.h"
#include "worldhandler.h"
#include "util/log.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

foxyBot::foxyBot()
{
	textcolour = TEXT_COLOUR;
	activities[0] = new ExploreActivity();
	activities[1] = new CraftActivity();
}

bool foxyBot::PostSetup()
{
	psengine->GetMsgHandler()->Subscribe(this, MSGTYPE_MODE);
	psengine->GetMsgHandler()->Subscribe(this, MSGTYPE_GUISKILL);
	psengine->GetMsgHandler()->Subscribe(this, MSGTYPE_SYSTEM);
	psengine->GetMsgHandler()->Subscribe(this, MSGTYPE_CRAFT_INFO);
	psengine->GetMsgHandler()->Subscribe(this, MSGTYPE_GUISTORAGE);
	psengine->GetMsgHandler()->Subscribe(this, MSGTYPE_GUIINVENTORY);
	psengine->GetMsgHandler()->Subscribe(this, MSGTYPE_PERSIST_ITEM);
	psengine->GetMsgHandler()->Subscribe(this, MSGTYPE_UPDATE_ITEM);

	textbox = dynamic_cast<pawsMessageTextBox*> (FindWidget("FoxyBotMainText"));
	textbox->Show();
	OutputMsg(csString("Foxy bot ready and waiting."));

	logger = new BotLogCSV(psengine->GetConfig(), psengine->GetVFS());

	for (int i = 0; i<activitiesCount; i++)
		activities[i]->Init(logger, 0, textbox, csString(""));

	return true;
}
foxyBot::~foxyBot()
{
    psengine->GetMsgHandler()->Unsubscribe(this, MSGTYPE_MODE);
    psengine->GetMsgHandler()->Unsubscribe(this, MSGTYPE_GUISKILL);
    psengine->GetMsgHandler()->Unsubscribe(this, MSGTYPE_SYSTEM);
    psengine->GetMsgHandler()->Unsubscribe(this, MSGTYPE_CRAFT_INFO);
    psengine->GetMsgHandler()->Unsubscribe(this, MSGTYPE_GUISTORAGE);
    psengine->GetMsgHandler()->Unsubscribe(this, MSGTYPE_GUIINVENTORY);
    psengine->GetMsgHandler()->Unsubscribe(this, MSGTYPE_PERSIST_ITEM);
	psengine->GetMsgHandler()->Unsubscribe(this, MSGTYPE_UPDATE_ITEM);

    for (int i=0; i<activitiesCount; i++)
    	delete activities[i];

    delete logger;
}

void foxyBot::OutputMsg(csString msg)
{
	if (textbox != NULL)
	{
		csString out = csString(">") + msg;
		textbox->Show();
		textbox->AddMessage(out.GetData(), textcolour);
	}
}

void foxyBot::HandleMessage( MsgEntry* me )
{
	for (int i=0;i<activitiesCount;i++)
		activities[i]->HandleMessage(me);
}

csString foxyBot::ExecuteCmd(csString cmd)
{
	if (cmd.StartsWith("reset"))
	{
		for (int i=0; i<activitiesCount; i++)
		{
			activities[i]->Reset();
			OutputMsg(csString("Activity " + activities[i]->GetActivityName()+" has been reset."));
		}
		return csString("Reset done.");
	}
	else if (cmd.StartsWith("stop"))
	{
		for (int i=0; i<activitiesCount; i++)
		{
			activities[i]->Stop();
			OutputMsg(csString("Activity " + activities[i]->GetActivityName()+" has been stopped."));
		}
		return csString("Activities stopped.");
	}

	WordArray words(cmd, false);

	if (words.GetCount()<2)
	{
		csString out = csString("Usage: /bot craft total_times craftDelay m/M \n /bot explore total_times size_of_line leavekeys(0 or 1) threads_for_small_claims maxDelay");
		OutputMsg(out);
		return out;
	}

	int i = 0;
	while (i < activitiesCount && !activities[i]->GetActivityName().CompareNoCase(words[0]))
		i++;

	if (i < activitiesCount)
	{
		BotActivity* ba = activities[i];
		if (ba->IsError())
			return csString("Requested activity is in error state. Kindly reset bot and try again.");

		if (ba->IsOngoing())
			return csString("Requested activity is already running. ");

		if (ba->IsStopped() || ba->IsReady() || ba->IsFinished())
		{
			int repeat = words.GetInt(1);
			csString tail = words.GetTail(2);

			ba->Init(logger, repeat, textbox, tail);
			ba->Start();

			return csString("Bot started requested activity.");
		}
		return csString("Requested activity is in unknown state, please reset bot and try again.");
	}

	return csString("Requested activity not found.");
}
