/*
 * worldhandler.cpp
 *
 *  Created on: 22 Jul 2015
 *      Author: Diana Coman
 */

#include <psconfig.h>
#include <csgeom/math3d.h>
#include "globals.h"

#include "psclientchar.h"
#include "gui/inventorywindow.h"
#include "gui/pawscontainerdescwindow.h"
#include "util/slots.h"

#include "worldhandler.h"
#include "cmdusers.h"

#include "pscharcontrol.h"


void worldHandler::ExecCmd(csString cmd)
{
	psengine->GetCmdHandler()->Execute(cmd);
}

void worldHandler::StepForward()
{
	const psMovement* fwd = psengine->GetCharControl()->GetMovementManager()->FindMovement("forward");
	psengine->GetCharControl()->GetMovementManager()->Push(fwd);
	psengine->GetCharControl()->GetMovementManager()->ControlledActor()->Movement().ExtrapolatePosition(0.3);
}

void worldHandler::StepBackward()
{
	const psMovement* fwd = psengine->GetCharControl()->GetMovementManager()->FindMovement("backward");
	psengine->GetCharControl()->GetMovementManager()->Push(fwd);
	psengine->GetCharControl()->GetMovementManager()->ControlledActor()->Movement().ExtrapolatePosition(0.3);
}

void worldHandler::StopAllMovement()
{
	psengine->GetCharControl()->GetMovementManager()->StopAllMovement();
}

void worldHandler::TurnRight(float rad)
{
	psengine->GetCelClient()->GetMainPlayer()->Rotate(0, rad, 0);
}

void worldHandler::TurnToPoint(float x, float z)
{
	csVector3 currPos;
    csVector3 gotoPos = csVector3(x, 0, z);
    float yRot;
    iSector *sector;
    psengine->GetCelClient()->GetMainPlayer()->GetLastPosition(currPos, yRot, sector);
    csVector3 diff = gotoPos - currPos;
    float targetYRot = atan2(-diff.x,-diff.z);

    psengine->GetCelClient()->GetMainPlayer()->Rotate(0, targetYRot, 0);
}

void worldHandler::Turn(float angle)
{
	csVector3 currPos;
	float yRot;
	iSector *sector;
	psengine->GetCelClient()->GetMainPlayer()->GetLastPosition(currPos, yRot, sector);
	yRot = yRot + angle;
	psengine->GetCelClient()->GetMainPlayer()->SetPosition(currPos, yRot, sector);
}


void worldHandler::SetRotation(float yRot)
{
	csVector3 currPos;
	float yRotOld;
	iSector *sector;
	psengine->GetCelClient()->GetMainPlayer()->GetLastPosition(currPos, yRotOld, sector);

	psengine->GetCelClient()->GetMainPlayer()->SetPosition(currPos, yRot, sector);
}

csVector3 worldHandler::GetCurrentPos()
{
	csVector3 currPos;
	float yRot;
	iSector *sector;
	psengine->GetCelClient()->GetMainPlayer()->GetLastPosition(currPos, yRot, sector);

	return currPos;
}

float worldHandler::GetCurrentRot()
{
	csVector3 currPos;
	float yRot;
	iSector *sector;
	psengine->GetCelClient()->GetMainPlayer()->GetLastPosition(currPos, yRot, sector);

	return yRot;
}

csPtr<iEventTimer> worldHandler::GetStandardTimer()//iObjectRegistry* object_reg)
{
	return csEventTimer::GetStandardTimer(psengine->GetObjectRegistry());
}


bool worldHandler::IsInBrain(csString itemName)
{
	pawsSlot* slot = FindInventoryItemSlot(itemName);
	if (slot == NULL)
		return false;
	else
		return (slot->GetID() == PSCHARACTER_SLOT_MIND);
}

pawsSlot* worldHandler::FindInventoryItemSlot(csString itemName)
{
	pawsInventoryWindow* window = (pawsInventoryWindow*)PawsManager::GetSingleton().FindWidget("InventoryWindow");
    pawsSlot* slot = dynamic_cast <pawsSlot*>(window->FindWidget(itemName.GetData()));

    return slot;
}

psInventoryCache::CachedItemDescription* worldHandler::FindItemSlot(csString itemName, bool anywhere)
{
	csArray<psInventoryCache::CachedItemDescription*>::Iterator iter = psengine->GetInventoryCache()->GetSortedIterator();
	psInventoryCache::CachedItemDescription* from = NULL;

	while (iter.HasNext())
	    		    {
	    		        psInventoryCache::CachedItemDescription* slotDesc = iter.Next();

	    		        if (slotDesc->name.CompareNoCase(itemName))
	    		        {
	    		            if (anywhere || (slotDesc->containerID == CONTAINER_INVENTORY_BULK))
	    		            {
	    		                from = slotDesc;
	    		                break;  // We found an item, so breaking the search
	    		            }
	    		        }
	    		    }
	return from;
}

bool worldHandler::MoveItems(int fromContainer, int fromSlot, int toContainer, int toSlot, int quantity)
{
	if (fromContainer == CONTAINER_INVENTORY_BULK)
	{
	  fromSlot -= PSCHARACTER_SLOT_BULK1;
	  fromContainer =  (INVENTORY_SLOT_NUMBER)(fromSlot/100);
	  fromSlot = fromSlot%100;
	}
	psSlotMovementMsg msg( fromContainer, fromSlot, toContainer, toSlot, quantity);
	msg.SendMessage();
	return true;
}

bool worldHandler::Equip(csString itemName)
{
	csString cmd = csString("/equip ") + itemName;
	psengine->GetCmdHandler()->Execute(cmd.GetData());
	return true;
}

bool worldHandler::Dequip(csString itemName)
{
	csString cmd = csString("/dequip ") + itemName;
	psengine->GetCmdHandler()->Execute(cmd.GetData());
	return true;
}

bool worldHandler::EquipRecipe(csString itemName)
{
	psInventoryCache::CachedItemDescription* slot = FindItemSlot(itemName, true);
	if (slot == NULL)
		return false;

	if (slot->containerID != CONTAINER_INVENTORY_EQUIPMENT)	//equip only if truly needed
	{
		MoveItems(slot->containerID, slot->slot, CONTAINER_INVENTORY_EQUIPMENT, PSCHARACTER_SLOT_MIND, slot->stackCount);
	}

	return true;
}

csString worldHandler::GetEquippedToolName()
{
	psInventoryCache::CachedItemDescription* slot = psengine->GetInventoryCache()->GetInventoryItem(PSCHARACTER_SLOT_RIGHTHAND);

	if (slot == NULL)
		return "";
	else return slot->name;
}

csString worldHandler::GetBrainItemName()
{
	psInventoryCache::CachedItemDescription* slot = psengine->GetInventoryCache()->GetInventoryItem(PSCHARACTER_SLOT_MIND);

	if (slot == NULL)
		return "";
	else return slot->name;

}

bool worldHandler::IsWithinReach(csString name)
{
	GEMClientObject* object = FindItemInWorld(name);

	if (object)
		return true;
	return false;
}

GEMClientObject* worldHandler::FindItemInWorld(csString name)
{
    psCelClient* cel = psengine->GetCelClient();
    csVector3 myPos = cel->GetMainPlayer()->GetPosition();
    GEMClientObject* bestObject = NULL;
    float distance = 0.0f;

    // Find all entities within a certain radius.
    csArray<GEMClientObject*> entities = cel->FindNearbyEntities(cel->GetMainPlayer()->GetSector(),
	                                                                 myPos,
	                                                                 NEARBY_TARGET_MAX_RANGE);

	// Iterate through the entity list looking for one with the right name.
	size_t entityCount = entities.GetSize();

	for (size_t i = 0; i < entityCount; ++i)
	{
	   // Get the next entity, skip if it's me or the starting entity.
	        GEMClientObject* object = entities[i];
	        CS_ASSERT( object );

	        if (!csString(object->GetName()).StartsWith(name, true))
	            continue;
	        csVector3 pos(object->GetPosition());
	        float seDistance = csSquaredDist::PointPoint(myPos, pos);
	        if (!bestObject || seDistance < distance)
	        {
	            bestObject = object;
	            distance = seDistance;
	        }
	 }
    return bestObject;
}

bool worldHandler::Target(csString containerName)
{
	csString cmd = csString("/target ") + containerName;
	psengine->GetCmdHandler()->Execute(cmd.GetData());
	return true;
}

bool worldHandler::TargetEID(EID eid)
{
	GEMClientObject * obj = psengine->GetCelClient()->FindObject(eid);
	if(obj)
	{
		psengine->GetCharManager()->SetTarget(obj, "select");
		return true;
	}
	else
		return false;
}

csString worldHandler::GetTargetName()
{
	csString name = csString("");

	GEMClientObject *object = NULL;
	object = psengine->GetCharManager()->GetTarget();

	if (object != NULL)
		name =  object->GetName();

	return name;
}

bool worldHandler::OpenTarget()
{
	GEMClientObject *object = NULL;
	object = psengine->GetCharManager()->GetTarget();

	if (object == NULL)
		return false;

	EID id = object->GetEID();

	psViewItemDescription out(id.Unbox(), -1);
	out.SendMessage();

	return true;
}

bool worldHandler::PicklockEID(EID eid)
{
	GEMClientObject * obj = psengine->GetCelClient()->FindObject(eid);
	if(obj)
	{
		psengine->GetCharManager()->SetTarget(obj, "select");
		psLockpickMessage lockpick("");
		lockpick.SendMessage();
		return true;
	}
	else
		return false;
}

bool worldHandler::OpenTargetEID(EID eid)
{
	GEMClientObject * obj = psengine->GetCelClient()->FindObject(eid);
	if (obj == NULL)
		return false;

	psViewItemDescription out(eid.Unbox(), -1);
	out.SendMessage();
	return true;
}

int worldHandler::GetTargetID()
{
	GEMClientObject *object = NULL;
	object = psengine->GetCharManager()->GetTarget();

	if (!object)
		return -1;

	return object->GetEID().Unbox();
}
bool worldHandler::CloseOpenWindow(csString windowName)
{
	csString cmd = csString("/show ") + windowName;
	psengine->GetCmdHandler()->Execute(cmd.GetData());
	return true;
}
bool worldHandler::CombineContentsInTarget()
{
	psengine->GetCmdHandler()->Execute("/combine");
	return true;
}

bool worldHandler::UseTarget()
{
	psengine->GetCmdHandler()->Execute("/use");
	return true;
}

bool worldHandler::TakeAllFromTarget()
{
	psengine->GetCmdHandler()->Execute("/takeall");
	//psengine->GetCharManager()->LockTarget(false);	//unlock target, just in case
	return true;
}

bool worldHandler::TrainSkillMax(csString skillName)
{
	if (!IsWithinReach(trainerName))
		return false;

	csString cmd = csString("/target ") + trainerName;

	psengine->GetCmdHandler()->Execute(cmd.GetData());
	psengine->GetCmdHandler()->Execute("/train");

	csString commandData;
	commandData.Format("<B NAME=\"%s\" AMOUNT=\"%d\"/>", EscpXML(skillName).GetData(), 100);
	psGUISkillMessage msg(psGUISkillMessage::BUY_SKILL, commandData);
	msg.SendMessage();

	return true;
}
