/*
 * worldhandler.h
 *
 *  Created on: 22 Jul 2015
 *      Author: Diana Coman
 */

#ifndef WORLDHANDLER_H_
#define WORLDHANDLER_H_

#include <csutil/timer.h>
#include "net/cmdhandler.h"
#include "gui/pawsslot.h"
#include "pscelclient.h"
#include "psinventorycache.h"

const csString trainerName = csString("Heina Draggenfort");

static const csString ITEM_NAME_FOR_TINY_CLAIMS = csString("Little Bit O' Nothing");
static const csString ITEM_NAME_FOR_SMALL_CLAIMS = csString("Coarse Frangible Thread");

//handlers for interaction with the game world and original client core
class worldHandler
{
	//auxiliary methods for item handling
public:

	//wrapper for any command to pass directly
	static void ExecCmd(csString cmd);

	//movement
	static void StepForward();
	static void StepBackward();
	static void StopAllMovement();
	static void TurnRight(float rad);
	static void TurnToPoint(float x, float z);
	static void Turn(float angle);
	static void SetRotation(float yRot);
	static csVector3 GetCurrentPos();
	static float GetCurrentRot();

	//timer (crystalspace)
	static csPtr<iEventTimer> GetStandardTimer();//iObjectRegistry* object_reg);

	//tests and finder methods
	static bool IsInBrain(csString itemName);
	static bool IsWithinReach(csString name);
    static pawsSlot* FindInventoryItemSlot(csString itemName);
    static psInventoryCache::CachedItemDescription* FindItemSlot(csString itemName, bool anywhere);


    //item action methods
    static bool MoveItems(int fromContainer, int fromSlot, int toContainer, int toSlot, int quantity);
    static bool Equip(csString itemName);
    static bool Dequip(csString itemName);
    static bool EquipRecipe(csString itemName);
    static csString GetEquippedToolName();
    static csString GetBrainItemName();

    //containers and world objects targeting/using/combining/opening/properties
    static bool Target(csString containerName);
    static bool TargetEID(EID eid);
    static csString GetTargetName();
    static bool OpenTarget();
    static bool OpenTargetEID(EID eid);
    static bool CombineContentsInTarget();
    static bool UseTarget();
    static bool PicklockEID(EID eid);
    static bool TakeAllFromTarget();
    static bool CloseOpenWindow(csString windowName);
    static GEMClientObject* FindItemInWorld(csString name);

    //training
    static bool TrainSkillMax(csString skillName);

    /**
     * Gets the ID of the currently targeted item.
     * Returns -1 if nothing is currently targeted.
     */
    static int GetTargetID();
};



#endif /* WORLDHANDLER_H_ */
